

#ifndef qobject_hookH
#define qobject_hookH

#include <qobject.h>
#include "qtypes.h"


class QObject_hook : public QObject {
  Q_OBJECT

  public:

  QObject_hook(QObject *handle) : QObject(NULL) {
    this->handle = handle;
    this->events.func = NULL;
    this->destroyed_event.func = NULL;
    connect(handle, SIGNAL(destroyed()), this, SLOT(destroyed_hook()));
  }

  virtual ~QObject_hook() {
    if (handle) {
      //printf("before calling potentially dangling pointer handle...\n");
      handle->removeEventFilter(this);
      //printf("after calling potentially dangling pointer handle.\n");
      handle->disconnect(this);
      handle = NULL;
    } //else printf("avoided dangling pointer problem\n");
  }

  void hook_events(QHook &hook) {
    if (handle) {
      if (!events.func) {
        handle->installEventFilter(this);
        events = hook;
      }
      if (!hook.func) handle->removeEventFilter(this);
      events = hook;
    }
  }

  void hook_destroyed(QHook &hook) {
    destroyed_event = hook;
  }

  protected:

    QObject *handle;

    virtual bool eventFilter(QObject *sender, QEvent *event) {
      if (events.func) {
        typedef bool (*func_type)(void *data, QObject *sender, QEvent *event);
        return (*(func_type)events.func)(events.data, sender, event);
      }
      else return false;
    }
  
  private slots:

    void destroyed_hook() {
      if ( destroyed_event.func ) {
        typedef void (*func_type)(void *data);
        (*(func_type)destroyed_event.func)(destroyed_event.data);
      }
      // To avoid using dangling pointer
      handle = NULL;
    }
  
  private:
    QHook events;
    QHook destroyed_event;
};

#endif
